
--[[----------------------------------------------------------------------------

WFLuaExerciseTest.lua

--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
___________________

Copyright 2021 Adobe
All Rights Reserved.

NOTICE: All information contained herein is, and remains
the property of Adobe and its suppliers, if any. The intellectual
and technical concepts contained herein are proprietary to Adobe
and its suppliers and are protected by all applicable intellectual
property laws, including trade secret and copyright laws.
Dissemination of this information or reproduction of this material
is strictly forbidden unless prior written permission is obtained
from Adobe.

------------------------------------------------------------------------------]]

-- WF Lua Exercise
-- Author: [jandrew 2021-1-21]

--[[----------------------------------------------------------------------------

--==============================================================================
INTRODUCTION
--------------------------------------------------------------------------------

Welcome to the Wichita Foundation Lua Exercise! The purpose of the following
exercise is to help you write your first Lua code. It's a learning exercise,
and the only thing that matters is whether or not you learn anything; please
take your time, and ask lots of questions!

--==============================================================================
PRE-EXERCISE REQUIRED READING
--------------------------------------------------------------------------------

Before attempting this exercise, please read through the Lua manual located at:

https://www.lua.org/manual/5.1/

You should also make sure that you able to build and run the test runner(s), and
have Halide set up for debugging. See the new-hire FAQ for more info:

https://wiki.corp.adobe.com/display/lightroom/Wichita+Foundation+New+Hire+FAQ

--==============================================================================
GETTING STARTED
--------------------------------------------------------------------------------

Below you'll find some simple tasks. The goal of this exercise is to complete
these tasks, and ensure that the unit test associated with this test file
is passing.

If you're stuck and unsure about a step, please reach out to a WF engineer.

------------------------------------------------------------------------------]]

--------------------------------------------------------------------------------
-- STEP 1: SET THE FOLLOWING VARIABLE TO TRUE.
--------------------------------------------------------------------------------

local setToTrue = false
	-- Setting this to true enables the unit tests associated with this exercise.

--------------------------------------------------------------------------------
-- STEP 2: Create a new git branch inside of the wf submodule for this exercise.
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
-- STEP 3: Open WFLuaExerciseTest.lua inside of Halide.
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
-- STEP 4: Build and run the test runner. Please use either the Mac, Windows,
-- or iOS test runner for the purpose of this exercise.
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
-- STEP 5: Within the test runner, search for WFLuaExerciseTest and run the
-- test suite. It should fail.
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
-- STEP 6: Complete the coding exercises below. Once completed, the tests within
-- the WFLuaExerciseTest test suite should pass.
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
-- STEP 7: Once you've completed the coding exercise, commit your changes and
-- push your branch git git.corp.adobe.com. Create a Pull Request containing
-- your changes, and mark it with a "DO NOT MERGE" label. 
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
-- STEP 8: Notify engineers of your PR within the #wf-standup channel. 
--------------------------------------------------------------------------------

--==============================================================================
-- START CODING EXERCISE
--------------------------------------------------------------------------------

-- Please read through and implement the following functions according to the
-- directions within:

--------------------------------------------------------------------------------

local function variableArgs( ... )

	-- Given a variable number of arguments, pack them into a array.
	
	-- Example:
	-- input: 1, 2, 3..N
	-- output: { 1, 2, 3..N }
	
end

--------------------------------------------------------------------------------

local function functionsAsArgs( fn )

	-- Given a function (fn) return an anonymous function that will call this fn.
	
end

--------------------------------------------------------------------------------

local function errorHandling( errorFn )

	-- Given a function that will throw an error, call the function, capture
	-- the error message, and return the error message gracefully.
	
end

--------------------------------------------------------------------------------

local function phoneNumberFormatting( phoneNumber )

	-- Given a 10 digit phone number, format the number as:
	-- (NXX)-NXX-XXXX
	
	-- Example:
	-- input: "3605675309"
	-- output: "(360)-567-5309"
	
end

--------------------------------------------------------------------------------

local function isArraySumEvenOrOdd( arr )

	-- Iterate through this array and add together all the values or the array.
	
	-- If the sum is even, return an array containing the original array, the sum, 
	-- and the string "even".
	
	-- If the sum is odd, return an array containing the original array, the sum, 
	-- and the string "odd".
	
	-- Example:
	-- input: { 1, 2, 3, 4, 5 }
	-- output: { { 1, 2, 3, 4, 5 }, 15, "odd" }
	
end

--------------------------------------------------------------------------------

local function dedupAndSortArray( arrayWithDuplicates )

	-- Given an array that contains duplicate entries, output an array that
	-- has the duplicate removes and is sorted by value.

	-- Example:
	-- input: { 1, 5, 1, 2, 9, 3, 3, 4, 5 }
	-- output: { 1, 2, 3, 4, 5, 9 }

end


--------------------------------------------------------------------------------

local function flattenArray( nestedArray )

	-- Take a nested array and return a single flattened array containing
	-- all values except nil.

	-- Example:
	-- input: { 1, { 2, 3, nil, 4 }, {}, 5 }
	-- output: { 1, 2, 3, 4, 5 }

end

--------------------------------------------------------------------------------

local function requireOutsideCode( fnName, n )

	-- Create a new lua file called WFLuaExerciseHelper.lua. This file should
	-- return a table containing two functions:
	
	-- meow( n ): Should return the string "meow" n times.
		-- Example n=3 output: "meow meow meow"
		
	-- bark( n ): Should return the string "woof" n times.
		-- Example n=3 output: "woof woof woof"
	
	-- Include WFLuaExerciseHelper within this function.
	
	-- Example:
	-- input: "meow", 3
	-- output: "meow moew meow"
	
end

--==============================================================================
-- END CODING EXERCISE: PLEASE DO NOT MODIFY ANY OF THE CODE BELOW THIS LINE
--------------------------------------------------------------------------------

local WFLuaExerciseTest = {}

--------------------------------------------------------------------------------

function WFLuaExerciseTest.rxTestVerifyExercise( rxTest ) 

	if not setToTrue then
		return
	end

	local packedArray = variableArgs( 1, 2, "A", 3, "B", "C" )

	rxTest:expectEqual( { 1, 2, "A", 3, "B", "C" }, packedArray )

	----------------------------------------------------------------------------

	local rand = math.random()
	
	local fn = function()
		return rand
	end
	
	local resultFn = functionsAsArgs( fn )

	rxTest:expectEqual( rand, resultFn() )

	----------------------------------------------------------------------------

	local uniqueStr = require 'KSIdGenerator'.uuidInLowercaseHex()

	local errorFn = function()
		assert( false, uniqueStr )
	end
	
	local resultErrorStr = errorHandling( errorFn )
	
	rxTest:expectEqual( true, not not string.match( resultErrorStr, uniqueStr ) )

	----------------------------------------------------------------------------
	
	local phoneNumber = "3609738155"

	rxTest:expectEqual( "(360)-973-8155", phoneNumberFormatting( phoneNumber ) )

	----------------------------------------------------------------------------

	local arr = { 1, 2, 3, 4, 5, 6, 7, 1 }

	rxTest:expectEqual( { arr, 29, "odd" }, isArraySumEvenOrOdd( arr ) )

	local arr2 = { 1, 2, 3, 4, 5, 6, 7 }

	rxTest:expectEqual( { arr2, 28, "even" }, isArraySumEvenOrOdd( arr2 ) )

	----------------------------------------------------------------------------

	local arrayWithDuplicates = { 3, 5, 3, 7, 9, 11, 11, 4, 11 }

	rxTest:expectEqual( { 3, 4, 5, 7, 9, 11 }, dedupAndSortArray( arrayWithDuplicates ) )

	----------------------------------------------------------------------------

	local nestedArray = { 1, { 2, 3, nil, 4 }, {}, 5, { "A", "B" }, {} }

	rxTest:expectEqual( { 1, 2, 3, 4, 5, "A", "B" }, flattenArray( nestedArray ) )

	----------------------------------------------------------------------------

	rxTest:expectEqual( "meow meow", requireOutsideCode( "meow", 2 ) )
	rxTest:expectEqual( "woof woof woof woof", requireOutsideCode( "bark", 4 ) )
	
end

--------------------------------------------------------------------------------

	-- WF.core.self_test
local WFRxTestCase = require 'WFRxTestCase'

return WFRxTestCase.wrapSuite( WFLuaExerciseTest )
